//
//  EmptyPage
//
//  Copyright (c) 2018 linhay - https://github.com/linhay
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE

import UIKit

public struct EmptyPage<Base> {
  public let base: Base
  public init(_ base: Base) {
    self.base = base
  }
}

public protocol EmptyPageCompatible {
  associatedtype CompatibleType
  var ep: CompatibleType { get }
}

public extension EmptyPageCompatible {
    var ep: EmptyPage<Self> { return EmptyPage(self) }
}


extension UIScrollView: EmptyPageCompatible { }

public struct EmptyPageRuntime {
  
  static let swizzingLayout: Void = {
    
    swizzing(sel: #selector(UIScrollView.layoutSubviews),
             of: #selector(UIScrollView.emptyPage_layoutSubviews),
             in: UITableView.self)
    
    swizzing(sel: #selector(UIScrollView.layoutIfNeeded),
             of: #selector(UIScrollView.emptyPage_layoutIfNeeded),
             in: UIScrollView.self)
  }()
  
  /// 替换 tableView 相关函数
  static let swizzingTableView: Void = {
    swizzing(sel: #selector(UITableView.insertRows(at:with:)),
             of: #selector(UITableView.emptyPage_insertRows(at:with:)),
             in: UITableView.self)
    
    swizzing(sel: #selector(UITableView.deleteRows(at:with:)),
             of: #selector(UITableView.emptyPage_deleteRows(at:with:)),
             in: UITableView.self)
    
    swizzing(sel: #selector(UITableView.insertSections(_:with:)),
             of: #selector(UITableView.emptyPage_insertSections(_:with:)),
             in: UITableView.self)
    
    swizzing(sel: #selector(UITableView.deleteSections(_:with:)),
             of: #selector(UITableView.emptyPage_deleteSections(_:with:)),
             in: UITableView.self)
    swizzing(sel: #selector(UITableView.reloadData),
             of: #selector(UITableView.emptyPage_reloadData),
             in: UITableView.self)
  }()
  
  /// 替换 CollectionView 相关函数
  static let swizzingCollectionView: Void = {
    swizzing(sel: #selector(UICollectionView.reloadData),
             of: #selector(UICollectionView.emptyPage_reloadData),
             in: UICollectionView.self)
    
    swizzing(sel: #selector(UICollectionView.insertItems(at:)),
             of: #selector(UICollectionView.emptyPage_insertItems(at:)),
             in: UICollectionView.self)
    
    swizzing(sel: #selector(UICollectionView.deleteItems(at:)),
             of: #selector(UICollectionView.emptyPage_deleteItems(at:)),
             in: UICollectionView.self)
    
    swizzing(sel: #selector(UICollectionView.insertSections(_:)),
             of: #selector(UICollectionView.emptyPage_insertSections(_:)),
             in: UICollectionView.self)
    
    swizzing(sel: #selector(UICollectionView.deleteSections(_:)),
             of: #selector(UICollectionView.emptyPage_deleteSections(_:)),
             in: UICollectionView.self)
  }()
  
  /// 交换方法
  ///
  /// - Parameters:
  ///   - selector: 被交换的方法
  ///   - replace: 用于交换的方法
  ///   - classType: 所属类型
  static func swizzing(sel: Selector,of: Selector,in classType: AnyClass) {
    let select1 = sel
    let select2 = of
    
    guard let select1Method = class_getInstanceMethod(classType, select1) else {
      assertionFailure("can't find method: " + select1.description)
      return
    }
    
    guard let select2Method = class_getInstanceMethod(classType, select2) else {
      assertionFailure("can't find method: " + select1.description)
      return
    }
    
    let didAddMethod = class_addMethod(classType,
                                       select1,
                                       method_getImplementation(select2Method),
                                       method_getTypeEncoding(select2Method))
    if didAddMethod {
      class_replaceMethod(classType,
                          select2,
                          method_getImplementation(select1Method),
                          method_getTypeEncoding(select1Method))
    }else {
      method_exchangeImplementations(select1Method, select2Method)
    }
  }
}
